starterControllers.controller('BasketCtrl',
	['$scope', '$q', 'DataSvc', '$state', '$ionicLoading', '$ionicPopup', '$timeout', '$filter',
	function($scope, $q, DataSvc, $state, $ionicLoading, $ionicPopup, $timeout, $filter) {

	_oz.deb('BasketCtrl init', $state.params);

	$scope.currentBasketLength = 0;

	$scope.hiddenTip = function(tipid) {
		return DataSvc.TipIsSeen(tipid);
	};

	$scope.hideTip = function(tipid) {
		DataSvc.TipMarkAsSeen(tipid);
	};


    $scope.getBasketContents = function() {
        var contentRequest = {}, deleteRequest = {};

        $scope.basketEntryList = [];
        $scope.basketTotal = 0;

        _.each($scope.basket.PaymentFlowReferenceList, function(flowRef) {
            contentRequest[flowRef] = DataSvc.GetPaymentFlowDetails(flowRef);
        });

        $q.all(contentRequest).then(function(results) {
            var toDelete = [], newEpid, newEntry;
            // _oz.deb('cr', results);

            //
            // Now tidy up the contents.
            // If any of these flow results do not exist, they are chopped from the active list
            // and removed from the basket
            //
            newEntry = results[$scope.newPaymentFlowReference];

            if (newEntry) {
                newEpid = newEntry.ID;
                // _oz.deb('newid', newEpid);
            }

            _.each(results, function(basketEntry, entryId) {
                if (basketEntry._jtiraResult === 'FAILURE') {
                    // straightforward missing flow references can go
                    toDelete.push(entryId);
                } else {
                    //
                    // If a new paymentflow reference has been added, and there's an existing
                    // reference for the same epid, chop it out
                    //
                    if (basketEntry.ID === newEpid && entryId !== $scope.newPaymentFlowReference) {
                        toDelete.push(entryId);
                    } else {
                        $scope.basketEntryList.push({
                            entryId: entryId,
                            basketEntry: basketEntry
                        });
                    }
                }
            });

            _oz.deb('toDelete', toDelete);

            if (toDelete.length > 0) {
                _.each(toDelete, function(deleteFlowRef) {
                    deleteRequest[deleteFlowRef] = DataSvc.RemoveBasketEntry($scope.basket.BasketId, deleteFlowRef);
                });

                $q.all(deleteRequest).then(function (deleteResult) {
                    _oz.deb('der', deleteResult);
                });
            }


            //
            // Now transform the full entry into something easily displayed and totalled
            //
            $scope.displayEntryList = DataSvc.makeDisplayEntry($scope.basketEntryList);
            $scope.basketTotal = 0;

            _.each($scope.displayEntryList, function(displayEntry) {
                $scope.basketTotal = $scope.basketTotal + displayEntry.entryCost;
            });

            $scope.basketLength = $scope.displayEntryList.length;
        });
    };


    $scope.removeEntry = function(flowRef) {
        DataSvc.RemoveBasketEntry($scope.basket.BasketId, flowRef).then(function(result) {
            $scope.retrieveBasket();
        });
    };

    $scope.retrieveBasket = function() {
        DataSvc.GetBasketCurrent().then(function(basketResult) {
            $scope.basket = basketResult.Basket;
            $scope.getBasketContents();
        });
    };


	$scope.doRefresh = function() {
		$scope.$broadcast('scroll.refreshComplete');

		$scope.refreshUserInfo();
	};


	$scope.payBasket = function() {
		var paymentDestination = _oz.setting.host() + '/JTIRA/JTIRA.aspx#!/ad-payment/' + $scope.basket.BasketId;

		// _oz.deb('window.open', paymentDestination);
					
		window.open(paymentDestination, '_system');

		$timeout(function () {
			$state.go('top');
		}, 5000);

		return;
	};

    DataSvc.GetBasketCurrent().then(function(basketResult) {
    	_oz.deb('basketResult', basketResult);
        
    	if (basketResult._jtiraResult !== 'SUCCESS') {
    		$state.go('top');
    		return;
    	}

        $scope.basket = basketResult.Basket;

        $scope.newPaymentFlowReference = $state.params.newPaymentFlowReference;

        if ($scope.newPaymentFlowReference) {
            DataSvc.AddBasketEntry($scope.basket.BasketId, $scope.newPaymentFlowReference).then(function(addFlowResult) {
                $scope.retrieveBasket();
            });
        } else {
            $scope.retrieveBasket();
        }

    });


}]);