starterControllers.controller('DetailedAdCtrl',
	['$scope', '$q', '$ionicLoading', '$ionicPopup', '$cordovaSocialSharing', '$ionicSideMenuDelegate', '$ionicScrollDelegate', '$state', 'DataSvc',
	function($scope, $q, $ionicLoading, $ionicPopup, $cordovaSocialSharing, $ionicSideMenuDelegate, $ionicScrollDelegate, $state, DataSvc) {

	_oz.deb('DetailedAdCtrl init', $state.params);

	$scope.vehicleDealerInfo = {};
	$scope.estateAgentInfo = {};
	$scope.propExtraLinks = [];

	$scope.categoryCode = 'GENERAL';

	$scope.contextListId = '';

	$scope.fatalError = false;


	$scope.doRefresh = function() {
		$scope.$broadcast('scroll.refreshComplete');

		$scope.loadAd();
	};


	$scope.hiddenTip = function(tipid) {
		return DataSvc.TipIsSeen(tipid);
	};

	$scope.hideTip = function(tipid) {
		DataSvc.TipMarkAsSeen(tipid);
	};

	$scope.showPics = function() {
		$state.go('pictureviewer', {epid: $scope.focusEpid});
	}

	$scope.showSideMenu = function(showMenu) {
		$ionicSideMenuDelegate.toggleLeft(showMenu);
	};

	$scope.gotoCategory = function(id) {
		$state.go('searchresults', {searchParameters: angular.toJson({CATMAIN: id})});
	};

	$scope.isFavouriteAd = function() {
		var favourite = false;

		if (!$scope.focusEpid) {
			return false;
		}

		if ($scope.userInfo && $scope.userInfo.IsLoggedIn) {
			if (_.contains($scope.userInfo.FAVOURITES, $scope.focusEpid)) {
				favourite = true;
			}
		};

		return favourite;
	};


	$scope.toggleFavourite = function() {
		DataSvc.ToggleFavourite($scope.focusEpid).then(function(result) {
			var alertPopup;

			if (result._jtiraResult === 'FAILURE') {
				alertPopup = $ionicPopup.alert({
			    	title: 'Login Required',
					template: 'Please login from the home screen to save your favourites.'
				});

				alertPopup.then(function(alertRes) {});
			}

			DataSvc.RefreshUserInformation(true).then(function(userInfo) {
				$scope.userInfo = userInfo;
				$scope.showSideMenu(false);
			});
		}, function(errorResult) {
			_oz.deb('toggleFavourite error', errorResult);
		});
	};

	$scope.favouriteText = function() {
		if ($scope.isFavouriteAd()) {
			return 'Unfavourite';
		} else {
			return 'Favourite';
		}
	};

	$scope.favouriteIcon = function() {
		if ($scope.isFavouriteAd()) {
			return 'ion-ios7-star';
		} else {
			return 'ion-ios7-star-outline';
		}
	};


	$scope.roomDesc = function(rooms, defaultText) {
		defaultText = defaultText || 'Unknown';

		if (!rooms) {
			return defaultText;
		}

		if (rooms === '' || rooms === '0' || rooms === 0) {
			return defaultText;
		} else {
			return rooms;
		}

	};

	$scope.defText = function(sourceText, defaultText) {
		defaultText = defaultText || 'Unknown';

		if (!sourceText) {
			return defaultText
		};

		if (sourceText.trim() === '') {
			return defaultText;
		} else {
			return sourceText;
		}
	};


	$scope.hasContactDetails = function() {

		if (!$scope.focusEpid) {
			return false;
		}

		if (!$scope.adDetail) {
			return false;
		}

		if ($scope.adDetail.FULL.Phone || $scope.adDetail.FULL.Mobile || $scope.adDetail.FULL.Email || $scope.adDetail.FULL.IsBulk) {
			return true;
		} else {
			return false;
		}
	};


	$scope.isAgency = function() {

		if (!$scope.adDetail) {
			return false;
		}

		if ($scope.adDetail.FULL.VehicleDealerCode || $scope.adDetail.FULL.JobAgencyCode || $scope.adDetail.FULL.AgentId) {
			return true;
		}

		return false;
	};

	$scope.isVehicleDealer = function() {
		if (!$scope.adDetail) {
			return false;
		}

		if ($scope.adDetail.FULL.VehicleDealerCode) {
			return true;
		}

		return false;
	};

	$scope.isJobAgency = function() {
		if (!$scope.adDetail) {
			return false;
		}

		if ($scope.adDetail.FULL.JobAgencyCode) {
			return true;
		}

		return false;
	};


	$scope.isEstateAgent = function() {
		if (!$scope.adDetail) {
			return false;
		}

		if ($scope.adDetail.FULL.AgentId) {
			return true;
		}

		return false;
	};

	$scope.openExternalLink = function(destinationUrl) {
		_oz.deb('openExternalLink', destinationUrl);

		window.open(destinationUrl, '_system');
	};

	$scope.dialTelephone = function(destinationNumber) {
		$scope.openExternalLink('tel:' + destinationNumber.replace(/[^0-9\*\+\#p\,]/g, ''));
	};

 
	$scope.sendNormalEmail = function(toAddress) {
		var subject = 'JT Insight Ad: ' + $scope.adDetail.FULL.Title;

		var body = 'Your ad: ' + _oz.setting.host() + $scope.adDetail.FULL.PageURL + '\r\n\r\n\r\n';

		$scope.sendEmail(toAddress, subject, body);
	};


	$scope.reportThisAd = function() {
		var toAddress = 'contact@jtinsight.com';

		var subject = 'REPORT AN AD - ' + $scope.adDetail.FULL.Title;

		var body =
			'[Please include the link below and supply any additional information]\r\n\r\n' +
			 _oz.setting.host() + $scope.adDetail.FULL.PageURL + '\r\n\r\n\r\n';

		$scope.sendEmail(toAddress, subject, body);
	};

	$scope.sendCarDealerEmail = function(toAddress) {
		var subject = 'JT Insight Ad: ' + $scope.adDetail.FULL.Title;

		var body = 'Your ad: ' + _oz.setting.host() + $scope.adDetail.FULL.PageURL + '\r\n\r\n';

		body = body + 'Your ref: ' + $scope.adDetail.FULL.VehicleId + '\r\n\r\n\r\n';

		$scope.sendEmail(toAddress, subject, body);
	};


	$scope.sendEstateAgentEmail = function(toAddress) {
		var subject = 'JT Insight Ad: ' + $scope.adDetail.FULL.Title;

		var body = 'Your ad: ' + _oz.setting.host() + $scope.adDetail.FULL.PageURL + '\r\n\r\n';

		$scope.sendEmail(toAddress, subject, body);
	};


	$scope.sendJobAgencyEmail = function(toAddress) {
		var subject = 'JT Insight Ad: ' + $scope.adDetail.FULL.Title + ($scope.adDetail.FULL.JobRef ? ' Ref:' + $scope.adDetail.FULL.JobRef : '');

		var body = 'Your ad: ' + _oz.setting.host() + $scope.adDetail.FULL.PageURL + '\r\n\r\n';

		$scope.sendEmail(toAddress, subject, body);
	};


	$scope.sendEmail = function(toAddress, subject, body) {
		var mailTo = 'mailto:' + toAddress;

		mailTo = mailTo + '?subject=' + escape(subject) + '&body=' + escape(body);

		$scope.openExternalLink(mailTo);
	};


	$scope.shareViaEmail = function() {
		var message, subject, toArr, bccArr, file;

		subject = 'JT Insight - ' + $scope.adDetail.FULL.Title;

		message = _oz.setting.host() + $scope.adDetail.FULL.PageURL + '\r\n\r\n';

		$cordovaSocialSharing.shareViaEmail(message, subject, toArr, bccArr, file).then(function(result) {
	      	// Success! 
    	}, function(err) {
	      	// An error occured. Show a message to the user
    	});
	};

	$scope.shareViaTwitter = function() {
		var message, image, link;

		message = 'JT Insight - ' + $scope.adDetail.FULL.Title;
		link = _oz.setting.host() + $scope.adDetail.FULL.PageURL;

		$cordovaSocialSharing.shareViaTwitter(message, null, link).then(function(result) {
			var alertPopup = $ionicPopup.alert({
		    	title: 'Share',
				template: 'twitter share success ' + result
			});
		}, function(err) {
			var alertPopup = $ionicPopup.alert({
		    	title: 'Share',
				template: 'twitter share failed ' + err
			});
		});

	};



	$scope.gotoPrevious = function() {
		$scope.moveToEpid($scope.epidPrev);
	};

	$scope.gotoNext = function() {
		$scope.moveToEpid($scope.epidNext);
	};

	$scope.scrollTop = function() {
		$ionicScrollDelegate.$getByHandle('detailedAdContent').scrollTop();
	};

	$scope.renderPage = function(adDetail) {
		var locationText;

		if (!adDetail) {
			$scope.fatalError = true;

			return;
		}

		$scope.adDetail = adDetail;

		$scope.picData = [];

    	switch ($scope.adDetail.catMain.Code) {
    		case 'VEHICLES':
    		case 'BOATS':
    			$scope.categoryCode = 'VEHICLES';
    			break;

    		case 'JOBS':
    			$scope.categoryCode = 'JOBS';
    			break;

    		case 'PROPERTY':
    			$scope.categoryCode = 'PROPERTY';
    			break;

    		default:
    			break;
    	}


		if ($scope.adDetail.DA === $scope.adDetail.DU) {
			$scope.adDate = _oz.dateTime($scope.adDetail.DA).toDateString();
			$scope.adDateLabel = "Created"
		} else {
			$scope.adDate = _oz.dateTime($scope.adDetail.DU).toDateString();
			$scope.adDateLabel = "Last updated"
		}

		$scope.headerPrefix = $scope.adDetail.formatted.price;
		$scope.headerTitle = $scope.adDetail.formatted.title;

    	if ($scope.categoryCode === 'GENERAL' || $scope.categoryCode === 'JOBS') {
			_.each($scope.adDetail.FULL.ImageList, function(im) {
				if (im.URL) {
					$scope.picData.push(_oz.absUrl(im.URL) + '?preset=det-main');
				}
			});
    	}


		if ($scope.categoryCode === 'VEHICLES') {

			$scope.adPriceHeader = $scope.adPriceHeader + ' - ' + ($scope.adDetail.FULL.Dealer ? 'Dealer' : 'private vendor');
			
			$scope.vehMakeAndModel = _oz.join([
				_oz.def($scope.adDetail.FULL.Make, 'make unknown'),
				_oz.def($scope.adDetail.FULL.Model, 'model unknown')]
			);

			$scope.vehYearAndMileage = _oz.join([
				_oz.def($scope.adDetail.FULL.Year, 'year unknown'),
				_oz.def($scope.adDetail.FULL.Mileage, 'mileage unknown')]
			);

			if ($scope.adDetail.FULL.VehicleDealerCode) {
				$scope.vehicleDealerInfo = $scope.lookup.carDealerList[$scope.adDetail.FULL.VehicleDealerCode];

				$scope.vehicleDealerInfo.Logo = _oz.absUrl($scope.vehicleDealerInfo.Logo);

				_.each($scope.adDetail.FULL.ImageList, function(im) {
					if (im.URL) {
						$scope.picData.push(_oz.absUrl(im.URL));
					}
				});
			} else {
				_.each($scope.adDetail.FULL.ImageList, function(im) {
					if (im.URL) {
						$scope.picData.push(_oz.absUrl(im.URL) + '?preset=det-main');
					}
				});
			}
		}


		if ($scope.categoryCode === 'PROPERTY') {
			console.log($scope.adDetail);

			if ($scope.adDetail.FULL.Location) {
				locationText = $scope.adDetail.FULL.Location;
				locationText = locationText.replace('Jersey: ', '');
				locationText = locationText.replace('Guernsey: ', '');

				$scope.headerPrefix = $scope.headerPrefix + ' ' + locationText;
			}

			if ($scope.adDetail.FULL.AgentId) {
				$scope.estateAgentInfo = $scope.lookup.estateAgentList[$scope.adDetail.FULL.AgentId];

				$scope.estateAgentInfo.Logo = _oz.absUrl($scope.estateAgentInfo.Logo);

				//
				// Extract any other documents or external links from embedded media
				//
				$scope.propExtraLinks = _.filter($scope.adDetail.FULL.Media.DocumentList, function(doc) {
					if (doc.Document && doc.Document !== '') {
						return doc;
					}
				});

				_oz.deb('propExtraLinks', $scope.propExtraLinks);
			};

			if ($scope.adDetail.FULL.AgentId && $scope.adDetail.FULL.AgentMediaPrefix) {
				_.each($scope.adDetail.FULL.Media.ImageList, function(propIm) {
					if (propIm.Image) {
						$scope.picData.push(_oz.absUrl($scope.adDetail.FULL.AgentMediaPrefix + propIm.Image));
					}
				});
				_.each($scope.adDetail.FULL.Media.FloorplanList, function(propIm) {
					if (propIm.Floorplan) {
						$scope.picData.push(_oz.absUrl($scope.adDetail.FULL.AgentMediaPrefix + propIm.Floorplan));
					}
				});

				$scope.picData = _.first($scope.picData, 4);

			} else {
				_.each($scope.adDetail.FULL.ImageList, function(im) {
					if (im.URL) {
						$scope.picData.push(_oz.absUrl(im.URL) + '?preset=det-main');
					}
				});
			}

			tenureData = _.find($scope.lookup.propertyData.Tenure, {ID: $scope.adDetail.FULL.Tenure});
			qualificationData = $scope.adDetail.PQUA;
			saleStatusData = _.find($scope.lookup.propertyData.SaleStatus, {ID: $scope.adDetail.FULL.SaleStatus});
			rentFrequencyData = _.find($scope.lookup.propertyData.RentFrequency, {ID: $scope.adDetail.FULL.RentFrequency});

			$scope.propAccomodation = _oz.join([
				_oz.elem('strong', $scope.adDetail.FULL.Bedrooms) + ' bedroom',
				_oz.def($scope.adDetail.FULL.Bathrooms, '') !== '' ? _oz.elem('strong', $scope.adDetail.FULL.Bathrooms) + ' bathroom' : '', 
				_oz.def($scope.adDetail.FULL.LivingRooms, '') !== '' ? _oz.elem('strong', $scope.adDetail.FULL.LivingRooms) + ' living room' : '', 
				_oz.def($scope.adDetail.FULL.ParkingSpaces, '') !== '' ? _oz.elem('strong', $scope.adDetail.FULL.ParkingSpaces) + ' parking spaces' : '', 
			]); 

			$scope.propQualAndTenure = _oz.join([
				qualificationData ? qualificationData : 'Qualifications unknown',
				tenureData ? tenureData.Description : 'Tenure unknown'
			]);

			$scope.propSaleStatusDescription = saleStatusData ? saleStatusData.Description : '';
			$scope.propRentFrequencyDescription = rentFrequencyData ? rentFrequencyData.Description : '';

			$scope.adPriceHeader = $scope.adPriceHeader + ' - ' + _oz.def(adDetail.FULL.Location, 'location not supplied');

			if ($scope.propSaleStatusDescription !== '') {
				$scope.adPriceHeader = $scope.propSaleStatusDescription + ': ' + $scope.adPriceHeader;
			}

		}


		if ($scope.categoryCode === 'JOBS') {
			if ($scope.adDetail.P === 0 || $scope.adDetail.P === '' || $scope.adDetail.P === '0') {
				$scope.adPriceHeader = 'Salary on application';
			}

			$scope.jobTermsAndNature = _oz.join([adDetail.JCON, adDetail.JNAT === true ? 'Full-time' : 'Part-time']);

			if ($scope.adDetail.FULL.JobAgencyCode) {
				$scope.jobAgencyInfo = $scope.lookup.jobAgencyList[$scope.adDetail.FULL.JobAgencyCode];

				$scope.jobAgencyInfo.Logo = _oz.absUrl($scope.jobAgencyInfo.Logo);
			};
		}
	};


	$scope.moveToEpid = function(epid) {
		var contextEpidList, contextIndex, navAdList = [], contextData;

		$scope.focusEpid = epid;

		$scope.contextList.paging.focusEpid = epid;

		DataSvc.SetContextData($scope.contextListId, $scope.contextList);

		contextEpidList = ($scope.contextList.cachedResultList.resultList) ? $scope.contextList.cachedResultList.resultList.split('|') : [];

		contextIndex = contextEpidList.indexOf($scope.focusEpid);

		$scope.epidPrev = $scope.epidNext = '';

		// we have a previous index
		if (contextIndex > 0) {
			$scope.epidPrev = contextEpidList[contextIndex - 1];
			$scope.adPrev = $scope.lookup.epidLookup[$scope.epidPrev];
			navAdList.push($scope.adPrev);
		}

		// we have a next 
		if (contextIndex < (contextEpidList.length - 1)) {
			$scope.epidNext = contextEpidList[contextIndex + 1];
			$scope.adNext = $scope.lookup.epidLookup[$scope.epidNext];
			navAdList.push($scope.adNext);
		}

		if (navAdList.length > 0) {
			DataSvc.PopulateMediumDetail(navAdList).then(function (result) {
			});
		}


		$ionicLoading.show(_oz.waiting('Fetching detail'));

		DataSvc.PopulateFullAd($scope.focusEpid).then(function(adDetail) {
			$scope.scrollTop();
			$ionicLoading.hide();

			if (adDetail && adDetail.id) {
				$scope.renderPage(adDetail);

				//
				// Update stats for this category
				//
				_oz.deb('UpdateCategoryStats', DataSvc.UpdateCategoryStats(adDetail.C2));
			} else {
				$scope.fatalError = true;
			}

		}, function(reason) {
			$ionicLoading.hide();
		});

		//
		// Grab the previous and next just in case to speed up page loading
		//
		_.each(navAdList, function(navEpid) {
			if (navEpid && navEpid.id) {
				DataSvc.PopulateFullAd(navEpid.id).then(function(adDetail) {
				});
			}
		});
	};


	$scope.loadAd = function() {
		$scope.fatalError = false;

		$ionicLoading.show(_oz.waiting('Fetching ad'))

		$q.all({
			lookup: DataSvc.GetLookups(),
			userInfo: DataSvc.RefreshUserInformation(true)
		}).then(function(results) {
			$scope.lookup = results.lookup;

			$scope.params = $state.params;

			$scope.userInfo = results.userInfo;

			$scope.contextListId = $scope.params.contextId;

			$scope.contextList = DataSvc.GetContextData($scope.contextListId);

			_oz.deb('my context is', $scope.contextList);

			$scope.focusEpid = $scope.contextList.paging.focusEpid;

			$scope.moveToEpid($scope.focusEpid);

		}, function(reason) {
	    	_oz.deb('$q DetailedAdCtrl error', reason);

			$ionicLoading.hide();

	    	$scope.fatalError = true;
	    });

	};

	$scope.loadAd();

}]);