starterControllers.controller('EditCtrl', function($scope, $ionicLoading, $cordovaCamera, $ionicPopup, $q, $state, $ionicTabsDelegate, DataSvc) {

	_oz.deb('EditCtrl init', $state.params);

	$scope.params = $state.params;

	$scope.displayImage = [];
	$scope.originalImage = [];
	$scope.originalImageCount = 0;

	$scope.focusEpid = $scope.params.epid || 'UNKNOWN';

	if ($scope.focusEpid === 'UNKNOWN') {
		$state.go('myaccount');

		return;
	};

	$scope.detailIcon = 'ion-compose';

	$scope.categoryCode = 'GENERAL';
	
	$scope.showDetails = function() {
		if ($scope.categoryCode === 'GENERAL' || $scope.categoryCode === 'SERVICES') {
			return false;
		} else {
			return true;
		}
	};

	$scope.onReady = function() {
	};

	$scope.contactBadgeContent = function() {
		if ($scope.isContactable() === false) {
			return '!';
		} else {
			return '';
		}
	};

	$scope.contactBadgeStyle = function() {
		if ($scope.isContactable() === false) {
			return 'badge-assertive';
		} else {
			return '';
		}
	};

	$scope.imageBadgeContent = function() {
		if (!$scope.epidAd) {
			return '0 ';
		}

		if ($scope.originalImageCount === 0) {
			return '?'
		} else {
			return $scope.originalImageCount;
		}
	};

	$scope.imageBadgeStyle = function() {
		if ($scope.originalImageCount === 0) {
			return 'badge-assertive';
		} else {
			return '';
		}
	};

	$scope.isHabitable = function() {
		if (!$scope.categoryCode === 'PROPERTY') {
			return false;
		}

		if (!$scope.editableData) {
			return false;
		}

		if (($scope.editableData.Property.Bedrooms && $scope.editableData.Property.Bedrooms !== '0') ||
			$scope.editableData.Property.Bathrooms ||
			$scope.editableData.Property.Livingrooms ||
			$scope.editableData.Property.ParkingSpaces) {
			return true;
		} else {
			return false;
		}
	};

	$scope.detailBadgeContent = function() {
		if (!$scope.epidAd || !$scope.categoryCode) {
			return '?';
		}

		if ($scope.categoryCode === 'PROPERTY' && !$scope.isHabitable()) {
			return '?';
		}

		return '';
	};

	$scope.detailBadgeStyle = function() {
		return 'badge-assertive';
	};


	$scope.takePhoto = function(source, elementIndex) {
		var alertPopup;

		var cameraPopoverOptions = {};

		var sourceType = (source === 'camera') ? Camera.PictureSourceType.CAMERA : Camera.PictureSourceType.PHOTOLIBRARY;

	    var theOptions = { 
	        quality : _oz.setting.defImageQuality(), 
	        allowEdit : false,
	        sourceType : sourceType,
	        destinationType: Camera.DestinationType.FILE_URL,
	        encodingType: Camera.EncodingType.JPEG,
	        correctOrientation: true,
	        targetWidth: _oz.setting.defImageWidth(),
	        targetHeight: _oz.setting.defImageHeight(),
	        popoverOptions: cameraPopoverOptions,
	        saveToPhotoAlbum: false
	    };

		try {
		    $cordovaCamera.getPicture(theOptions).then(function(imageReturn) {

				var photoPreview = document.getElementById('_oz_ID_IMAGE' + elementIndex);
		    	var picData;

				photoPreview.src = imageReturn;
				
				$scope.displayImage[elementIndex].dirty = true;
		    }, function(err) {
				_oz.deb('camera err', err);

				alertPopup = $ionicPopup.confirm({
			    	title: 'CAMERA ERR(1)',
					template: 'Err:' + err
				});

				alertPopup.then(function(alertRes) {});
		    });
		}
		catch (ex) {
			alertPopup = $ionicPopup.confirm({
		    	title: 'CAMERA ERR(2)',
				template: 'Ex:' + ex
			});

			alertPopup.then(function(alertRes) {});
		}
	};
	

	$scope.persistDirtyImages = function(destination) {
		_.each($scope.displayImage, function(imageEntry, index) {
			if (imageEntry.dirty === true) {
				imageEntry.data = $scope.getData(index)
			}
		});

		$scope.saveAndMoveOn(destination);
	};

	$scope.getData = function(imageIndex) {
		var original, theCanvas, ctx, theData;

		try {
			original = document.getElementById('_oz_ID_IMAGE' + imageIndex);
			theCanvas = document.createElement('canvas');
			theCanvas.width = original.naturalWidth;
			theCanvas.height = original.naturalHeight;
			ctx = theCanvas.getContext('2d');
			ctx.drawImage(original, 0, 0, original.naturalWidth, original.naturalHeight);
			theData = theCanvas.toDataURL('image/jpeg');

			return theData;
		}
		catch (ex) {
			return 'XXXX' + ex;
		}
	};


	$scope.isContactable = function() {
		if (!$scope.editableData) {
			return false;
		}

		if (!$scope.editableData.Common.Mobile && !$scope.editableData.Common.Phone && !$scope.editableData.Common.Email) {
			return false;
		} else {
			return true;
		}
	};

	$scope.calculateAdCost = function() {
		var adCost = 0;

		var currentRate = $scope.pendingRate.categoryPricing.RateList[$scope.pendingRate.categoryPricingIndex];

		adCost = currentRate.Standard;

		if ($scope.pendingRate.extras.Spotlight === true) {
			adCost = adCost + currentRate.Spotlight;
		}

		if ($scope.pendingRate.extras.Featured === true) {
			adCost = adCost + currentRate.Featured;
		}

		return adCost;
	};

	$scope.saveAndMoveOn = function(destination, params) {
		var saveAd, imageList;

		saveAd = _oz.assembleAd($scope.editableData, $scope.categoryCode, $scope.epidAd.JTIRAPricing.pendingRate);

		imageList = $scope.epidAd.ImageList;

		$ionicLoading.show(_oz.waiting('Saving ad'));

		DataSvc.SaveClassified(saveAd, imageList).then(function(saveResponse) {
			_oz.deb('SaveClassified', saveResponse);

			var saveImageList;

			$ionicLoading.hide();

			//
			// Now we check to see if any images have been changed
			// and save them accordingly
			//
			saveImageList = _.filter($scope.displayImage, {dirty: true}) ;

			$ionicLoading.show(_oz.waiting('Saving images'));

			$scope.saveImagesAndMoveOn(saveImageList, destination, params);				
		});
	};


	$scope.saveImagesAndMoveOn = function(remainingSaveList, destination, params) {
		var imageData, imageEntry;

		if (remainingSaveList.length === 0) {
			$ionicLoading.hide();

			if (destination) {
				$state.go(destination, params);
			}

			return;
		}

		imageEntry = _.first(remainingSaveList);

		$ionicLoading.show('Saving image ' + imageEntry.originalIndex + 1);

		DataSvc.SaveBase64Image(imageEntry.data, 'appimage.jpg', $scope.focusEpid, imageEntry.originalIndex).then(function(saveResult) {

			if (saveResult._jtiraResult === 'SUCCESS') {
				$scope.displayImage[imageEntry.originalIndex].url = _oz.setting.host() + saveResult.ImageUrl;
			}

			$scope.saveImagesAndMoveOn(_.rest(remainingSaveList), destination);

		});

	};

	$scope.publishAd = function() {
		$scope.saveAndMoveOn('publish-confirm', {epid: $scope.focusEpid});
	};

	$scope.hideKeyboard = function() {
		Keyboard.close();
	};

	$scope.adState = function() {
		var state, expiryDateTime, now;

		if (!$scope.epidAd) {
			return '';
		}

		now = new Date();
        expiryDateTime = _oz.dateTime($scope.epidAd, 'StopPublish');

		if ($scope.epidAd.ClassifiedState === 'SavedAndWaitingForPayment') {
			state = 'draft';
		} else {
			if (now > expiryDateTime) {
				state = 'expired';
			} else {
				state = 'live';
			}
		}

		return state;

	};

	$scope.publishTitle = function() {
		if ($scope.adState() === 'draft') {
			return 'Publish';
		} else {
			return 'Renew';
		}
	};

	$scope.deleteAd = function() {
		var alertPopup, title, template;

		switch ($scope.adState()) {
			case 'draft':
				title = 'Delete draft ad';
				template = 'You are about to delete your draft ad. Are you sure?'
				break;

			case 'live':
				title = 'Delete live ad';
				template = 'You are about to delete your live ad. This cannot be undone. Are you sure?'
				break;

			case 'expired':
				title = 'Delete expired ad';
				template = 'You are about to delete your expired ad. Are you sure?'
				break;

			default:
				return;
		}

		alertPopup = $ionicPopup.confirm({
	    	title: title,
			template: template,
			okText: 'DELETE',
			okType: 'button-assertive',
			cancelText: 'No',

		});

		alertPopup.then(function(res) {
			if (res !== true) {
				return;
			}

			$ionicLoading.show(_oz.waiting('Deleting ad'));

			DataSvc.DeleteClassified($scope.focusEpid).then(function(deleteResult) {
				_oz.deb('DeleteClassified', $scope.focusEpid, deleteResult);

				$ionicLoading.hide();

				$state.go('myaccount');
			});
		});
	};

	$scope.selectPublish = function() {
		$ionicTabsDelegate.select(4)
	};

	$scope.spotlightValues = function() {
		var theValues = [];

		theValues.push({
			description: 'SPOTLIGHT!',
			value: true
		});

		theValues.push({
			description: 'No spotlight',
			value: false
		});

		return theValues;
	};

	$scope.selectTab = function(index) {
		if (index === 1) {
			//
			// Skip the detail tab if there's no detail
			//
			if (!$scope.showDetails()) {
				index = index + 1;
			}
		}

    	$ionicTabsDelegate.select(index);
  	}

	$ionicLoading.show(_oz.waiting('Fetching ad'));


    $q.all({
    	lookup: DataSvc.GetLookups(),
    	epidAd: DataSvc.MyClassifiedDetail($scope.focusEpid, true)
    }).then(function(results) {
    	var epidAd, locationData = [], tenureData = [], qualificationData = [] ;

    	// _oz.deb('EditCtrl results', results);

    	$ionicLoading.hide();

    	epidAd = results.epidAd;

    	if (results.epidAd._jtiraResult === 'FAILURE') {
    		$state.go('top');
    		return;
    	}

    	$scope.lookup = results.lookup;
    	$scope.epidAd = epidAd;
    	$scope.catMain = $scope.lookup.category.idToCategory[$scope.epidAd.CatMain];
    	
    	switch ($scope.catMain.Code) {
    		case 'VEHICLES':
    		case 'BOATS':
    			$scope.detailIcon = 'ion-model-s';
    			$scope.categoryCode = 'VEHICLES';
    			break;

    		case 'JOBS':
    			$scope.detailIcon = 'ion-person-stalker';
    			$scope.categoryCode = 'JOBS';
    			break;

    		case 'PROPERTY':
    			$scope.detailIcon = 'ion-home';
    			$scope.categoryCode = 'PROPERTY';
    			break;

    		default:
    			break;
    	}

    	//
    	// Calculate the applicable current pricing structure
    	//
    	$scope.pendingRate = $scope.epidAd.JTIRAPricing.pendingRate;

    	//
    	// Set the default no-image display from the main
    	// category code
    	//
    	_.each([0, 1, 2, 3], function(picIndex) {
	    	$scope.displayImage.push({
	    		originalIndex: picIndex,
	    		data: '',
	    		dirty: false,
	    		url: _oz.setting.host() + '/images/assets/no-image-' + $scope.epidAd.CatMain + '.png'
	    	});
    	});

    	//
    	// Now paste in any src that already exists
    	//
    	$scope.originalImageCount = 0;
    	_.each($scope.epidAd.ImageList, function(imageEntry, destinationIndex) {
    		if (imageEntry.URL) {
    			$scope.displayImage[destinationIndex].url = _oz.setting.host() + imageEntry.URL;
    			$scope.originalImageCount = $scope.originalImageCount + 1;
    		}
    	});

    	$scope.originalImage = _.cloneDeep($scope.displayImage);

		//
		// Transform property lookups as required
		//

        //
        // Locations are nested by default. flatten to a list of "Main: Sub"
        // this is how locations are currently represented on production
        //
        _.each(results.lookup.propertyData.Location, function(subLocationList, mainLocation) {
            _.each(subLocationList, function(subLocation) {
                //
                // The '(Any)' selection is allowed only for queries, not initial placement
                //
                if (subLocation !== '(Any)') {
                    locationData.push(mainLocation + ': ' + subLocation);
                }
            });
        });

        $scope.locationData = locationData;

        _.each(results.lookup.propertyData.Qualification, function(entry) {
            if (entry.Description !== 'Any') {
                qualificationData.push(entry.Description);
            }
        });

        $scope.qualificationData = qualificationData;

        _.each(results.lookup.propertyData.Tenure, function(entry) {
            if (entry.DisplayEntry === true) {
                tenureData.push(entry);
            }
        });

        $scope.tenureData = tenureData;

    	_oz.deb('epidAd', $scope.epidAd);

		$scope.editableData = {
			Common: {
				ID: $scope.focusEpid.split('-')[1],
				Title: epidAd.Title,
				Description: epidAd.Description,
				Mobile: epidAd.Mobile,
				Phone: epidAd.Phone,
				Email: epidAd.Email,
				Price: epidAd.Price
			},
			Property: {
				Bedrooms: epidAd.Bedrooms || 0,
				Bathrooms: epidAd.Bathrooms || 0,
				Livingrooms: epidAd.Livingrooms || 0,
				ParkingSpaces: epidAd.ParkingSpaces || 0,
				Parish: epidAd.Location || $scope.locationData[0],
				Qualification: epidAd.Qualification || $scope.qualificationData[0],
				Tenure: epidAd.Tenure || $scope.tenureData[0].ID
			},
			Jobs: {
				Salary: epidAd.Salary,
				Location: epidAd.JobLocation,
				AgencyDetails: epidAd.Agency,
				ReferenceNumber: epidAd.JobRef,
				Nature: (epidAd.Fulltime === true) ? 'FullTime' : 'PartTime'
			},
			Vehicles: {
				Make: epidAd.Make,
				Model: epidAd.Model,
				Year: epidAd.Year,
				DealerOrPrivate: (epidAd.Dealer === true ) ? 'Dealer' : 'Private',
				Mileage: epidAd.Mileage
			}
		};

    	$scope.onReady();
    }, function(reason) {
    	_oz.deb('$q EditCtrl error', reason);
    });

});