starterControllers.controller('MyAccountCtrl',
	['$scope', '$q', 'DataSvc', '$state', '$ionicLoading', '$ionicPopup', '$ionicListDelegate', '$filter',
	function($scope, $q, DataSvc, $state, $ionicLoading, $ionicPopup, $ionicListDelegate, $filter) {

	_oz.deb('MyAccountCtrl init', $state.params);

	$scope.loginStateIcon = 'ion-log-in';
	$scope.loginTitle = 'Login to JT Insight';
	$scope.loginState = false;
	$scope.loginPrompt = 'Login';


	$scope.loginDetail = {
		username: '',
		password: ''
	};

	$scope.userAds = [];
	$scope.userSearches = [];

	$scope.userInformation = {};

	$scope.favouriteAds = [];

	$scope.messageList = [];

	$scope.currentBasketLength = 0;

	$scope.hiddenTip = function(tipid) {
		return DataSvc.TipIsSeen(tipid);
	};

	$scope.hideTip = function(tipid) {
		DataSvc.TipMarkAsSeen(tipid);
	};


	$scope.adNote = function(ad) {
		return _oz.adNote(ad);
	};


	$scope.logoutUser = function() {
		$ionicLoading.show(_oz.waiting('Logging out'));

		DataSvc.Logout().then(function(result) {
			$ionicLoading.hide();

			$state.go('top');
		});
	};

	$scope.favouriteLength = function() {
		if ($scope.userInformation.FAVOURITES && $scope.userInformation.FAVOURITES.length > 0) {
			return $scope.userInformation.FAVOURITES.length
		} else {
			return ;
		}
	};

	$scope.removeFavourite = function(ad) {
		DataSvc.ToggleFavourite(ad.id).then(function(result) {
			DataSvc.RefreshUserInformation(true).then(function(userInfo) {
				$scope.userInformation = userInfo;
				$scope.filterFavouriteAds();
				$ionicListDelegate.closeOptionButtons();
			});
		}, function(errorResult) {
			_oz.deb('ToggleFavourite errorResult', errorResult);
		});
	};

	$scope.userAdLength = function() {
		if ($scope.userAds) {
			return $scope.userAds.length;
		} else {
			return 0;
		}
	};


	$scope.userAdBadgeStyle = function() {
		if ($scope.expiredAds && $scope.expiredAds.length > 0) {
			return 'badge-assertive';
		} else {
			return '';
		}
	};


	$scope.userSearchLength = function() {
		if ($scope.userSearches) {
			return $scope.userSearches.length;
		} else {
			return 0;
		}
	};



	$scope.filterFavouriteAds = function() {
		var missing;

		$scope.favouriteAds = [];

		missing =  _.remove($scope.userInformation.FAVOURITES, function(favEpid) {
			var entry = $scope.lookup.epidLookup[favEpid];

			if (entry) {
				$scope.favouriteAds.push(entry);
				return false;
			} else {
				return true;
			}
		});

		return missing;
	};

	$scope.persistListState = function(focusEpid) {
		var contextData = DataSvc.GetContextData('FAVOURITES');

		contextData.paging.focusEpid = focusEpid;
		contextData.cachedResultList.resultList = $scope.userInformation.FAVOURITES.join('|');

		DataSvc.SetContextData('FAVOURITES', contextData);
	};


	$scope.refreshBasket = function() {
	    DataSvc.GetBasketCurrent().then(function(basketResult) {
	        _oz.deb('basketResult', basketResult);

	        $scope.currentBasketLength = 0;

	        if (basketResult.Basket) {
	            $scope.currentBasketLength = basketResult.Basket.PaymentFlowReferenceList.length;

	            _oz.deb('basketLength', $scope.currentBasketLength);
	        }
	    });
	};


	$scope.refreshUserInfo = function() {
		$ionicLoading.show(_oz.waiting('Fetching user details'));

		$q.all({
			userInformation: DataSvc.RefreshUserInformation(true),
			lookup: DataSvc.GetLookups()
		}).then(function(result) {
			var missing = [];

			$ionicLoading.hide();

			$scope.lookup = result.lookup;
			$scope.userInformation = result.userInformation;

			if ($scope.userInformation.IsLoggedIn !== true) {
				$state.go('top');

				return;
			}

			$scope.changePasswordHref = _oz.setting.host() + '/JTIRA/JTIRA.aspx#!/login/login/' + $scope.userInformation.UserId;

			missing = $scope.filterFavouriteAds();

			//
			// Populate user's saved searches
			//
			_.forOwn($scope.userInformation.SEARCHLIST, function(value, key) {
				$scope.userSearches.push({
					description: key,
					params: _oz.fromWebParams(value)
				});
			});

			$ionicLoading.show(_oz.waiting('Fetching favourites'));

			DataSvc.PopulateMediumDetail($scope.favouriteAds).then(function(result) {

				$ionicLoading.show(_oz.waiting('Fetching ads'));

				DataSvc.GetUserAds(true).then(function(userAds) {
			        var now = new Date(), liveAds = [];

					$ionicLoading.hide();

			        if (userAds._jtiraResult === 'FAILURE') {
			        	$scope.messageList.push('Failed to retrieve Ads [' + userAds._jtiraLogRef + ']');

			        	return ;
			        }

					$scope.userAds = userAds;

					//
					// Format in the same way as a standard ad,
					// and sort into draft, prod and expired buckets
					//
			        $scope.draftAds = [];
			        $scope.expiredAds = [];

			        _.each($scope.userAds, function(ad) {
			            var expiryDateTime = _oz.dateTime(ad.FULL, 'StopPublish');


				        ad.formatted = {};

		        		ad.formatted.price = $filter('currencyInt')(ad.MIN.P, '£');
		        		ad.formatted.title = $filter('drama')(ad.FULL.Title);

						if (ad.MED.Image) {
							ad.formatted.listImage = _oz.absUrl(ad.MED.Image + '?preset=app-list-sm');
						} else {
							ad.formatted.listImage = _oz.absUrl('/images/assets/no-image-' + ad.MIN.C1 + '.png?preset=app-list-sm');
						}

			            if (ad.FULL.ClassifiedState === 'SavedAndWaitingForPayment') {
			                $scope.draftAds.push(ad);
			            } else {
			                if (now > expiryDateTime) {
			                    $scope.expiredAds.push(ad);
			                } else {
			                    liveAds.push(ad);
			                }
			            }
			        }, function(reason) {
			        	_oz.deb('GetUserAds error', reason);
			        });

			        //
			        // Order so that the first to expire comes first
			        //
			        $scope.liveAds = _.sortBy(liveAds, function(entry) {return entry.FULL.StopPublish;});
				});
			});

			$scope.refreshBasket();
			
		}, function(reason) {
	    	_oz.deb('$q refreshUserInfo error', reason);
	    });
	};

	$scope.doRefresh = function() {
		$scope.$broadcast('scroll.refreshComplete');

		$scope.refreshUserInfo();
	};

	$scope.refreshUserInfo();

}]);