starterControllers.controller('PostQuickCtrl', function($scope, $ionicLoading, $ionicPopup, $cordovaCamera, $q, $state, DataSvc) {

	_oz.deb('PostQuickCtrl init', $state.params);

	$scope.imageAdded = false;

	$scope.chosenCategory = {
		filter: ''
	};

	$scope.quickDetails = {
		title: '',
		description: ''
	};

	$scope.mess = [];

	$scope.takePhoto = function(source, elementId) {
		var cameraPopoverOptions = {};

		var sourceType = (source === 'camera') ? Camera.PictureSourceType.CAMERA : Camera.PictureSourceType.PHOTOLIBRARY;

	    var theOptions = { 
	        quality : _oz.setting.defImageQuality(), 
	        allowEdit : false,
	        sourceType : sourceType,
	        destinationType: Camera.DestinationType.FILE_URL,
	        encodingType: Camera.EncodingType.JPEG,
	        correctOrientation: true,
	        targetWidth: _oz.setting.defImageWidth(),
	        targetHeight: _oz.setting.defImageHeight(),
	        popoverOptions: cameraPopoverOptions,
	        saveToPhotoAlbum: true
	    };

		$scope.mess.push('image options', theOptions);

	    $cordovaCamera.getPicture(theOptions).then(function(imageReturn) {
			$scope.mess.push('post getPicture');

			var photoPreview = document.getElementById(elementId);
	
			$scope.mess.push('filereturn', imageReturn);

			photoPreview.src = imageReturn;
			
			$scope.imageAdded = true;
		  
	    }, function(err) {
	      // An error occured. Show a message to the user
			$scope.mess.push('camera err', err);
			$scope.mess.push(err);
	    })
	};

	$scope.getData = function(elementId) {
		var original, theCanvas, ctx, theData;

		try {

			$scope.mess.push('getdata');

			original = document.getElementById(elementId);
			theCanvas = document.createElement("canvas");
			theCanvas.width = original.naturalWidth;
			theCanvas.height = original.naturalHeight;
			ctx = theCanvas.getContext("2d");
			ctx.drawImage(original, 0, 0, original.naturalWidth, original.naturalHeight);
			theData = theCanvas.toDataURL('image/jpeg');

			return theData;
		}
		catch (ex) {
			$scope.mess.push(ex);

			return '';
		}
	};

	$scope.saveAndContinue = function() {
		var mainCat, pendingRate;

		//
		// Check that an appropriate category and default
		// pricing has been selected, save and then
		// move on to edit the new Ad
		//
		adCategory = {
			mainCategoryId: $state.params.mainCatId,
			subCategoryId:$state.params.subCatId
		};

		//
		// Determine the rate list from the category code
		//
		mainCat = $scope.lookup.category.idToCategory[adCategory.mainCategoryId];

		pendingRate = {
			categoryPricing: $scope.lookup.pricingData.Pricing[mainCat.Code]
		};

		if (pendingRate.categoryPricing.Inherit) {
			pendingRate.categoryPricing = $scope.lookup.pricingData.Pricing[pendingRate.categoryPricing.Inherit];
		}

		pendingRate.categoryPricingIndex = pendingRate.categoryPricing.RateList.length - 1;


		//
		// Record the ratelist, selected rate, and any extras selected
		// In this case for a new quick ad, we just default to the most
		// expensive of any options, assumed to be the last in the list of tariffs
		//
		adPrice = {
			isNew: true,
			categoryPricing: pendingRate.categoryPricing,
			categoryPricingIndex: pendingRate.categoryPricing.RateList.length - 1,
			extras: {
				Spotlight: true,
				Featured: true
			},
			discounts: {
			}
		};

		adDetail = {
			Common: {
				Price: 0,
				Title: $scope.quickDetails.title,
				Description: $scope.quickDetails.description
			}
		};

		//
		// No images at this point, we default to the image representatice
		// of this main category and them paste in any chosen image over the top
		// The image URL is relative and this will be made absolute
		// when re-rendered from the server at a later date
		//
		adImages = [];

		_.times(4, function(n) {
			adImages.push({
				ImageIndex: n,
				ImageUrl: '/images/assets/no-image-' + $state.params.mainCatId + '.png'
			});
		});

		//
		// Save the draft first.
		// Then save any image selected, attaching it to the newly saved ad.
		// Finally move on to the full blown editing state
		//
		$ionicLoading.show(_oz.waiting('Saving your ad'));

		DataSvc.CreateClassified(
			adCategory,
			adPrice,
			adDetail,
			adImages
		).then(function(createResponse) {
			var alertPopup;
			var imageData;
			var epid;

			_oz.deb('CreateClassified createResponse', createResponse);

			$ionicLoading.hide();

			if (createResponse._jtiraResult !== 'SUCCESS') {
				//
				// Special processing if this looks like an authentication issue
				//
				if (createResponse._jtiraResultCode === 'DENIED') {
					alertPopup = $ionicPopup.alert({
				    	title: 'Failed to save ad',
						template: 'Sorry, we were unable to save your ad, as your login has timed out'
					});

					alertPopup.then(function(res) {
						$state.go('top');
					});
				} else {
					alertPopup = $ionicPopup.alert({
				    	title: 'Failed to save ad',
						template: 'Sorry, we were unable to save your ad [' + createResponse._jtiraResultCode + ']'
					});

					alertPopup.then(function(res) {
						$state.go('myaccount');
					});

				}

				return;
			}

			epid = 'epid-' + createResponse._jtiraPageData.ID;

			if ($scope.imageAdded == true) {
				$ionicLoading.show(_oz.waiting('Adding your picture'));

				imageData = $scope.getData('_oz_ID_MAINIMAGE');

				DataSvc.SaveBase64Image(imageData, 'appimage.jpg', epid, 0).then(function(saveResult) {
					_oz.deb('SaveBase64Image saveResult', saveResult);
					$ionicLoading.hide();

					$scope.mess.push(saveResult)

					alertPopup = $ionicPopup.alert({
				    	title: 'Ad Saved!',
						template: 'We\'ve saved your ad and picture, and it will now appear in your Drafts.<br>Continue to add more detail and publish.'
					});

					alertPopup.then(function(res) {
						$state.go('edit', {epid:epid});
					});

				});
			} else {

				alertPopup = $ionicPopup.alert({
			    	title: 'Ad Saved!',
					template: 'We\'ve saved your ad, and it will now appear in your Drafts.<br>Continue to add more detail and publish.'
				});

				alertPopup.then(function(res) {
					$state.go('edit', {epid:epid});
				});

			}
		});


	};


	$scope.onReady = function() {
		$ionicLoading.show(_oz.waiting('Checking user'));

		DataSvc.RefreshUserInformation(true).then(function(userInformation) {
			_oz.deb('RefreshUserInformation userInformation', userInformation);

			$ionicLoading.hide();

			$scope.userInformation = userInformation;

			if ($scope.userInformation.IsLoggedIn !== true) {
				$state.go('top', {destinationState: 'post-quickcatfilter'});

				return;
			}
		});
	};

	$scope.allowSave = function() {
		var allowed = false;

		if ($scope.quickDetails.title.length > 3 && $scope.quickDetails.description.length > 3) {
			allowed = true;
		}

		return allowed;
	};


	$ionicLoading.show(_oz.waiting('Initialising'));

    $q.all({
    	lookup: DataSvc.GetLookups()
    }).then(function(results) {
    	$ionicLoading.hide();

    	$scope.lookup = results.lookup;
    	$scope.defaultImage = _oz.setting.host() + '/images/assets/no-image-' + $state.params.mainCatId + '.png';
    	$scope.onReady();
    }, function(reason) {
    	_oz.deb('$q PostQuickCtrl error', reason);
    });

});