starterControllers.controller('SearchResultsCtrl',
	['$scope', '$q', '$ionicLoading', '$ionicSideMenuDelegate', '$ionicScrollDelegate', '$state', 'DataSvc',
	function($scope, $q, $ionicLoading, $ionicSideMenuDelegate, $ionicScrollDelegate, $state, DataSvc) {

	_oz.deb('SearchResultsCtrl init', $state.params);

	$scope.contextListId = $state.params.contextId;

	$scope.pageLength = _oz.setting.defPageLength();
	$scope.pageCurrent = 0;
	$scope.pageMax = 1;
	$scope.currentlyDisplayedPage ;

	$scope.showCategorySelector = false;
	$scope.showLocationSelector = false;

	$scope.locationList = [];
	$scope.tenureList = [];

	$scope.extraFilterCode = '';

	$scope.contextData = DataSvc.GetContextData($scope.contextListId);

	_oz.deb('contextData', $scope.contextData);

	$scope.contextData.filterSettings.sort = $scope.contextData.filterSettings.sort || 'FEATURED';


	$scope.doRefresh = function() {
		$scope.$broadcast('scroll.refreshComplete');

		$scope.refreshSearchResults();
	};

	$scope.hiddenTip = function(tipid) {
		return DataSvc.TipIsSeen(tipid);
	};

	$scope.hideTip = function(tipid) {
		DataSvc.TipMarkAsSeen(tipid);
	};

	$scope.sortFeatured = function() {
		if ($scope.contextData.filterSettings.sort !== 'FEATURED') {
			$scope.contextData.filterSettings.sort = 'FEATURED';
			$scope.contextData.paging.current = 0
			$scope.pageCurrent = 0;

			$scope.filterAndSort($scope.fullResultAdList, $scope.contextData.filterSettings);
		}
	};

	$scope.sortDate = function() {
		if ($scope.contextData.filterSettings.sort !== 'DACTIVE') {
			$scope.contextData.filterSettings.sort = 'DACTIVE';
		} else {
			$scope.contextData.filterSettings.sort = 'AACTIVE';
		}

		$scope.contextData.paging.current = 0
		$scope.pageCurrent = 0;

		$scope.filterAndSort($scope.fullResultAdList, $scope.contextData.filterSettings);
	};

	$scope.sortPrice = function() {
		if ($scope.contextData.filterSettings.sort !== 'APRICE') {
			$scope.contextData.filterSettings.sort = 'APRICE';
		} else {
			$scope.contextData.filterSettings.sort = 'DPRICE';
		}

		$scope.contextData.paging.current = 0
		$scope.pageCurrent = 0;

		$scope.filterAndSort($scope.fullResultAdList, $scope.contextData.filterSettings);
	};

	$scope.filterOnPrice = function() {
		$scope.debouncedRefreshFilter();
	};

	$scope.refreshFilter = function() {
		$scope.pageCurrent = 0;
		$scope.contextData.paging.current = 0;
		$scope.filterAndSort($scope.fullResultAdList, $scope.contextData.filterSettings);
	};

	$scope.debouncedRefreshFilter = _.debounce($scope.refreshFilter, 125);

	$scope.toggleCategorySelector = function() {
		$scope.showCategorySelector = !$scope.showCategorySelector;
	};


	$scope.categorySelectorHeading = function() {
		var heading = "Sub category";

		if ($scope.extraFilterCode === 'JOBS') {
			heading = 'Job type';
		}

		if ($scope.extraFilterCode === 'PROPERTY') {
			heading = "For sale / rent / other";
		}

		return heading;
	};

	$scope.setSubCategory = function(id) {
		$scope.contextData.filterSettings.CATSUB = id;
		$scope.filterAndSort($scope.fullResultAdList, $scope.contextData.filterSettings);
		$scope.toggleCategorySelector();
		$ionicSideMenuDelegate.toggleLeft(false);
	};

	$scope.subCategoryTitle = function() {
		if ($scope.contextData.filterSettings.CATSUB) {
			return $scope.lookup.category.idToCategory[$scope.contextData.filterSettings.CATSUB].Description;
		} else {
			return 'Any';
		}
	};

	$scope.adNote = function(ad) {
		return _oz.adNote(ad);
	};

	$scope.isFeatured = function(ad) {
		return ad.MIN.F === 'Y';
	};

	$scope.toggleLocationSelector = function() {
		$scope.showLocationSelector = !$scope.showLocationSelector;
	};

	$scope.setPropertyLocation = function(loc) {
		loc = loc || '';
		
		$scope.propertyLocation = loc;
		$scope.contextData.filterSettings.PLOC = loc.value;

		$scope.contextData.paging.current = 0
		$scope.pageCurrent = 0;
			
		$scope.filterAndSort($scope.fullResultAdList, $scope.contextData.filterSettings);
		$scope.toggleLocationSelector();
	};

	$scope.propertyLocationTitle = function() {
		var locationTitle = 'Any';

		if ($scope.contextData.filterSettings.PLOC) {
			locationTitle = $scope.contextData.filterSettings.PLOC;
		}

		return locationTitle;
	};

	$scope.bedroomTitle = function() {
		var title = 'Bedrooms';

		if ($scope.contextData.filterSettings.PBED && $scope.contextData.filterSettings.PBED > 0) {
			title = title + ': ' + $scope.contextData.filterSettings.PBED;
		}

		return title;
	}

	$scope.toggleMenu = function(showMenu) {
		$ionicSideMenuDelegate.toggleLeft(showMenu);
	};

	$scope.resetFilter = function() {
		var resetValues = {
			LUPD: '',
			PMIN: '', PMAX: '',
			VDLR: '', VMAK: '', VMOD: '', VMIL: '', VYER: '',
			JNAT: '', JLOC: '', JSAL: '',
			PAGN: '', PSUB: '', PLOC: '', PQUA: '', PTEN: '', PBED: '', PBAT: '', PLIV: '', PPAR: '',
			USER: ''
		};

		_.merge($scope.contextData.filterSettings, resetValues);

		$scope.filterAndSort($scope.fullResultAdList, $scope.contextData.filterSettings);


	};

	$scope.filterAndSort = function(sourceList, params) {
		var filteredResultList;
        var LUPD;
        var PMIN, PMAX;
        var VDLR, VMAK, VMOD, VYER, VMIL;
        var JNAT, JLOC, JSAL;
        var PAGN, PLOC, PQUA, PTEN, PBED, PBAT, PLIV, PPAR, PSUB;
        var USER;
        var contextEpidList = [];

        LUPD = (params.LUPD) ? Number(params.LUPD) : '';

        PMIN = (params.PMIN) ? params.PMIN : 0;
        PMAX = (params.PMAX) ? params.PMAX : 1000000000;

        VDLR = (params.VDLR) ? params.VDLR : '';
        VMAK = (params.VMAK) ? params.VMAK.toUpperCase() : '';
        VMOD = (params.VMOD) ? params.VMOD.toUpperCase() : '';
        VMIL = (params.VMIL) ? params.VMIL : '';
        VYER = (params.VYER) ? params.VYER : '';

        if (VMIL !== '') {
            VMIL = parseInt(VMIL, 10);

            if (isNaN(VMIL)) {
                VMIL = 10000000;
            }
        }

        if (VYER !== '') {
            VYER = parseInt(VYER, 10);

            if (isNaN(VYER)) {
                VYER = 10000000;
            }
        }


        JNAT = (params.JNAT) ? params.JNAT : '';

        JLOC = (params.JLOC) ? params.JLOC.toUpperCase() : '';

        JSAL = (params.JSAL) ? params.JSAL : '';

        if (JSAL !== '') {
            JSAL = parseInt(JSAL, 10);

            if (isNaN(JSAL)) {
                JSAL = 0;
            }
        }


        PAGN = (params.PAGN) ? params.PAGN : '';
        PSUB = (params.PSUB) ? params.PSUB : '';
        PLOC = (params.PLOC) ? params.PLOC.toUpperCase() : '';
        PQUA = (params.PQUA) ? params.PQUA.toUpperCase() : '';
        PTEN = (params.PTEN) ? params.PTEN : '';

        PBED = (params.PBED) ? params.PBED : '0';
        PBAT = (params.PBAT) ? params.PBAT : '0';
        PLIV = (params.PLIV) ? params.PLIV : '0';
        PPAR = (params.PPAR) ? params.PPAR : '0';

        PBED = parseInt(PBED, 10);
        PBAT = parseInt(PBAT, 10);
        PLIV = parseInt(PLIV, 10);
        PPAR = parseInt(PPAR, 10);

        USER = (params.USER) ? params.USER : '';

		filteredResultList = _.filter(sourceList, function(ad) {
			var EM = ad;
			var makeAndModel;
			var matchYear;

			//
			// Primary and secondary category Id
			//
			if (params.CATMAIN) {
				if (!(params.CATMAIN === EM.C1 || params.CATMAIN === EM.C2)) {
					return false;
				}
			}

			if (params.CATSUB) {
				if (!(params.CATSUB === EM.C2)) {
					return false;
				}
			}

            // Common
            if (EM.P > PMAX || EM.P < PMIN) {
                return false;
            }

            //
            // Vehicles
            //
            if ($scope.extraFilterCode === 'VEHICLES') {
                if (VDLR !== '') {
                    // VDLR is only present if true
                    if ((VDLR === 'N' && EM.VDLR) || (VDLR === 'Y' && !EM.VDLR)) {
                        return false;
                    }
                }

                makeAndModel = (EM.VMAK + ' ' + EM.VMOD).toUpperCase();

                if (VMAK !== '') {
                    if (!EM.VMAK || (makeAndModel.indexOf(VMAK) === -1)) {
                        return false;
                    }
                }

                // Model is variable so match on any substring
                if (VMOD !== '') {
                    if (!EM.VMOD || (makeAndModel.indexOf(VMOD) === -1)) {
                        return false;
                    }
                }

                // Mileage is a maximum, and only applied if present
                if (VMIL !== '') {
                    if (EM.VMIL) {
                        mileage = parseInt(EM.VMIL.replace(/\D/g, ''), 10);

                        if (!isNaN(mileage)) {
                            if (mileage > VMIL) {
                                return false;
                            }
                        }
                    }
                }

                // Year is also a maximum
                if (VYER !== '') {
                    if (EM.VYER) {
                    	//
                    	// Favour 4 digit years
                    	//
                    	matchYear = (EM.VYER.replace(/O/gi, '0')).match(/(19|20)\d\d/g);

                    	//
                    	// But settle for two
                    	//
                    	if (!matchYear) {
	                    	matchYear = (EM.VYER.replace(/O/gi, '0')).match(/\d\d/g);
                    	}

                    	if (matchYear) {
	                        regYear = parseInt(matchYear[0], 10);

	                        if (regYear < 1930) {
	                        	if (regYear < 30) {
	                        		regYear = regYear + 2000;
	                        	} else {
	                        		regYear = regYear + 1900;
	                        	}
	                        }

	                        if (!isNaN(regYear)) {
	                            if (regYear < VYER) {
	                                return false;
	                            }
	                        }
                    	}
                    }
                }
            }

            //
            // JOBS
            //
            if ($scope.extraFilterCode === 'JOBS') {
	            if (JNAT !== '') {
	                if (JNAT === 'F' && EM.JNAT !== true) {
	                    return false;
	                }

	                if (JNAT === 'P' && EM.JNAT === true) {
	                    return false;
	                }
	            }

	            // rough match on location
	            if (JLOC !== '') {
	                if (EM.JLOC && (EM.JLOC.toUpperCase().indexOf(JLOC) === -1)) {
	                    return false;
	                }
	            }

	            // minimum salary
	            if (JSAL !== '') {
	                if (EM.JSAL) {
	                    salary = parseInt(EM.JSAL, 10);

	                    if (!isNaN(salary)) {
	                        if (salary < JSAL) {
	                            return false;
	                        }
	                    }
	                }
	            }
            }

            //
            // PROPERTY
            //
            if ($scope.extraFilterCode === 'PROPERTY') {
	            if (PAGN !== '') {
	                if ((PAGN === 'N' && EM.PAGN) || (PAGN === 'Y' && !EM.PAGN)) {
	                    return false;
	                }
	            }

	            if (PSUB !== '') {
	                if (PSUB === 'RENT' && EM.C2 !== 51) {
	                    return false;
	                }

	                if (PSUB === 'SALE' && EM.C2 !== 53) {
	                    return false;
	                }
	            }

	            // exact matches are likely, but settle for a subset JIC
	            if (PLOC !== '') {
	                if (EM.PLOC && (EM.PLOC.toUpperCase().indexOf(PLOC) === -1)) {
	                    return false;
	                }
	            }

	            // match on exact quals, or ANY quals
	            if (!(PQUA === '' || PQUA === 'ANY')) {
	                if (EM.PQUA && EM.PQUA !== '') {
	                    if (!(EM.PQUA.toUpperCase() === PQUA || EM.PQUA.toUpperCase() === 'ANY')) {
	                        return false;
	                    }
	                }
	            }

	            if (PTEN !== '') {
	                if (EM.PTEN && EM.PTEN !== PTEN) {
	                    return false;
	                }
	            }

	            if (PBED > 0) {
	                if (EM.PBED) {
	                    bedrooms = parseInt(EM.PBED, 10);

	                    if (!isNaN(bedrooms)) {
	                        if (PBED > bedrooms) {
	                            return false;
	                        }
	                    }
	                }
	            }

	            if (PBAT > 0) {
	                if (EM.PBAT && PBAT > EM.PBAT) {
	                    return false;
	                }
	            }

	            if (PLIV > 0) {
	                if (EM.PLIV && PLIV > EM.PLIV) {
	                    return false;
	                }
	            }

	            if (PPAR > 0) {
	                if (EM.PPAR && PPAR > EM.PPAR) {
	                    return false;
	                }
	            }
            }

            return true;
		});

		$scope.sortedResultSet = DataSvc.SortResults(filteredResultList, params.sort)

		//
		// On each sort and filter, the context of the list is persisted
		// This allows detailed views to make decisions about how to
		// Navigate back and forth in the list
		//

		contextEpidList = _.pluck($scope.sortedResultSet, 'id').join('|');

		$scope.contextData.cachedResultList.resultList = contextEpidList;
		$scope.contextData.cachedResultList.lastUpdated = Date.now();

		//
		// If this is the first time in, set the paging up
		//
		if (!$scope.contextData.paging.current) {
	        $scope.pageCurrent = 0;
	        $scope.pageMax = Math.ceil($scope.sortedResultSet.length / _oz.setting.defPageLength()) ;

	        $scope.contextData.paging.current = $scope.pageCurrent;
	        $scope.contextData.paging.max = $scope.pageMax;
		} else {
	        $scope.pageCurrent = $scope.contextData.paging.current;
	        $scope.pageMax = $scope.contextData.paging.max ;
		}

		DataSvc.SetContextData($scope.contextListId, $scope.contextData);

		$scope.pageTitle = "Ads: " + $scope.sortedResultSet.length;

		$scope.renderPage();
	};

	
	$scope.renderPage = function() {
		//
		// Now work out which page full we take
		//
		var offset = $scope.pageCurrent * _oz.setting.defPageLength();

		$scope.currentlyDisplayedPage = $scope.sortedResultSet.slice(offset);

		$scope.currentlyDisplayedPage = _.take($scope.currentlyDisplayedPage, _oz.setting.defPageLength());

		DataSvc.PopulateMediumDetail($scope.currentlyDisplayedPage).then(function(mediumResult) {

		});

		$scope.contextData.paging.current = $scope.pageCurrent;

		DataSvc.SetContextData($scope.contextId, $scope.contextData);

		$scope.scrollTop();
	};


	$scope.nextPage = function() {
		$scope.pageCurrent = $scope.pageCurrent + 1;

		$scope.renderPage();
	};

	$scope.prevPage = function() {
		$scope.pageCurrent = $scope.pageCurrent - 1;

		$scope.renderPage();
	};

	$scope.showPager = function() {
		if ($scope.sortedResultSet && ($scope.sortedResultSet.length > _oz.setting.defPageLength())) {
			return true;
		} else {
			return false;
		}
	};

	$scope.scrollTop = function() {
		$ionicScrollDelegate.$getByHandle('resultsList').scrollTop();
	};

	$scope.enabledNext = function() {

	};

	$scope.enablePrev = function() {
		if ($scope.sortedResultSet && ($scope.pageCurrent > 0)) {
			return true;
		} else {
			return false;
		}
	};

	$scope.enableNext = function() {
		if ($scope.sortedResultSet && ($scope.pageCurrent < $scope.pageMax - 1)) {
			return true;
		} else {
			return false;
		}
	}

	$scope.persistListState = function(epid) {
		//
		// Record the current set of filters / position etc. so that they can be restored without issue
		// when navigating backwards and forwards.
		//
		$scope.contextData.paging.current = $scope.pageCurrent;
		$scope.contextData.paging.focusEpid = epid;

		DataSvc.SetContextData($scope.contextListId, $scope.contextData);
	};


	$scope.refreshSearchResults = function() {

		$q.all({
			lookup: DataSvc.GetLookups(),
			fullResultList: DataSvc.GetCachedResultList($scope.contextData.originalParameters.term)
		}).then(function(results) {
			var jerseyLocations = [], guernseyLocations = [], offIslandLocations = [];

			$scope.lookup = results.lookup;

			if ($scope.contextData.filterSettings.CATMAIN) {
				$scope.categoryMain = $scope.lookup.category.idToCategory[$scope.contextData.filterSettings.CATMAIN];

				if ($scope.categoryMain.parentCatId) {
					$scope.isMainCategory = false;
					$scope.parentCategory = $scope.lookup.category.idToCategory[$scope.categoryMain.parentCatId];
				} else {
					$scope.isMainCategory = true;

					$scope.subCatList = _.find($scope.lookup.category.simpleTree, {id: $scope.contextData.filterSettings.CATMAIN}).subCategoryList;

					// _oz.deb('scatlist', $scope.subCatList);
					// _oz.deb($scope.subCatList);
				}

				if ($scope.categoryMain.Code) {
					$scope.categoryCode = $scope.categoryMain.Code;
				} else {
					$scope.categoryCode = $scope.parentCategory.Code;
				}

				switch ($scope.categoryCode) {
					case 'VEHICLES':
					case 'CARS':
					case 'BOATS':
					case 'MOTORBIKES':
						$scope.extraFilterCode = 'VEHICLES';
						break;

					case 'JOBS':
						$scope.extraFilterCode = 'JOBS';
						break;

					case 'PROPERTY':
					case 'PROPERTY_FORSALE':
						$scope.extraFilterCode = 'PROPERTY';
						break;

					default:
						$scope.extraFilterCode = '';
						break;
				}
			}


			$scope.fullResultAdList = [];

			_.each(results.fullResultList, function(epid) {
				var ad = $scope.lookup.epidLookup[epid];

				if (ad) {
					$scope.fullResultAdList.push(ad);
				}
			});


			//
			// Transform location into easily rendered nested format
			//
			_.each($scope.lookup.propertyData.Location.Jersey, function(loc) {
				//
				// SKip '(Any)'
				//
				if (loc.indexOf('(') !== 0) {
					jerseyLocations.push({
						description: loc,
						value: 'Jersey: ' + loc
					});
				}
			});

			_.each($scope.lookup.propertyData.Location.Guernsey, function(loc) {
				//
				// SKip '(Any)'
				//
				if (loc.indexOf('(') !== 0) {
					guernseyLocations.push({
						description: loc,
						value: 'Guernsey: ' + loc
					});
				}
			})

			_.each($scope.lookup.propertyData.Location['Off-Island'], function(loc) {
				//
				// SKip '(Any)'
				//
				if (loc.indexOf('(') !== 0) {
					offIslandLocations.push({
						description: loc,
						value: 'Off-Island: ' + loc
					});
				}
			})

			$scope.locationList = {
				'Jersey': jerseyLocations,
				'Guernsey': guernseyLocations,
				'OffIsland': offIslandLocations
			};


			$scope.tenureList = _.filter($scope.lookup.propertyData.Tenure, {DisplayFiltering:true});

			$scope.qualificationList = $scope.lookup.propertyData.Qualification;

			$scope.fullResultList = results.fullResultList;

			$scope.filterAndSort($scope.fullResultAdList, $scope.contextData.filterSettings);
		}, function(reason) {
	    	_oz.deb('$q SearchResultsCtrl error', reason);
	    });
	}

	$scope.refreshSearchResults();

}]);