starterControllers.controller('TopCtrl', function($scope, $q, $state, DataSvc, $window, $timeout, $ionicPopup, $ionicLoading, $cordovaDevice) {

	var s = _oz.setting.restoreSettings();

	var now, lastRefresh;

	_oz.deb('TopCtrl init', $state.params);

	// _oz.tView('TOP');
	// _oz.tEvent('TCAT', 'TACTION', 'TLABEL', 0);

	$scope.fatalError = false;

	$scope.quickCat = [];

	$scope.searchParameters = {
		term: ''
	};

	$scope.maxSpotlight = _oz.setting.defSpotlight();

	$scope.isLoggedIn = false;

	$scope.browseTitle = function() {
		var searchTerm = $scope.searchParameters.term.trim();

		if (searchTerm) {
			return 'Search for "' + searchTerm + '" in';
		} else {
			return 'Browse categories';
		}
	};

	$scope.isSearching = function() {
		var searchTerm = $scope.searchParameters.term.trim();

		if (searchTerm) {
			return true;
		} else {
			return false;
		}

	};

	$scope.persistSpotlight = function(epid) {
		var contextId = 'SPOTLIGHT';

		var contextData = DataSvc.GetContextData();

		contextData.paging.focusEpid = epid;

		contextData.cachedResultList.resultList = _.pluck($scope.spotlightMatch, 'id').join('|');

		DataSvc.SetContextData('SPOTLIGHT', contextData);
	};

    $scope.persistAndGo = function(categoryId) {
        var contextId = 'QUICKSEARCH';

        var contextData = DataSvc.GetContextData();
		
		contextData.originalParameters = $scope.searchParameters;

        contextData.filterSettings.CATMAIN = categoryId;

        DataSvc.SetContextData(contextId, contextData);

        if (!categoryId && !$scope.searchParameters.term) {
        	$state.go('choosemaincat');
        } else {
	        $state.go('searchresults', {contextId:contextId});
        }
    };

	$scope.externalData = function() {
		return _oz.setting.externalData;
	};


	$scope.isProduction = function() {
		return _oz.setting.host() === 'https://www.jtinsight.com';
	};

	$scope.switchEnvironment = function(environment) {
		$ionicLoading.show(_oz.waiting('Switching'));

		_oz.setting.host() = _oz.setting.envList()[environment];

		DataSvc.Logout().then(function(result) {
			$ionicLoading.hide();
			DataSvc.ResetData();
		});
	};

	$scope.adNote = function(ad) {
		return _oz.adNote(ad);
	};

	$scope.myAccountPrompt = function() {
		if ($scope.isLoggedIn === true) {
			return 'My account';
		} else {
			return 'Login';
		}
	};

	$scope.loginAndGo = function(destination) {
		var myPopup;

		if ($scope.isLoggedIn === true) {
			$state.go(destination);

			return;
		}

		$state.go('login', {destination: destination});

	};


	$scope.doRefresh = function() {
		$scope.$broadcast('scroll.refreshComplete');

		DataSvc.ResetData();
		
		$scope.forceReload();
	};

	$scope.forceReload = function() {
		_oz.setting.lastRefresh(+new Date);

        $timeout(function() {
            _oz.deb('TopCtrl forceReload');
            $window.location.reload(true);
        }, 100);
	};


	$scope.retrieveData = function() {
		$ionicLoading.show(_oz.waiting('Fetching ads'));

		$q.all({
			userInfo: DataSvc.RefreshUserInformation(true),
			lookups: DataSvc.GetLookups()
		}).then(function(results) {
			var carCat, jobCat, propertyForSaleCat, numCat = 0;

			$ionicLoading.hide();

			$scope.isLoggedIn = results.userInfo.IsLoggedIn;
			$scope.lookups = results.lookups;

			$scope.categoryStats = DataSvc.GetCategoryStats();

			//
			// First time in, seed with known values
			//
			if ($scope.categoryStats.topCategoryList.length < 3) {
				carCat = $scope.lookups.category.codeToCategory.CARS;
				jobCat = $scope.lookups.category.codeToCategory.JOBS;
				propertyForSaleCat = $scope.lookups.category.codeToCategory.PROPERTY_FORSALE;

				$scope.categoryStats.topCategoryList.push(
					carCat.ID,
					propertyForSaleCat.ID,
					jobCat.ID
				);

				$scope.categoryStats.accumulatedStats[carCat.ID] = 5;
				$scope.categoryStats.accumulatedStats[propertyForSaleCat.ID] = 4;
				$scope.categoryStats.accumulatedStats[jobCat.ID] = 3;

				$scope.categoryStats.numSamples = 1;

				DataSvc.SetCategoryStats($scope.categoryStats);

				// _oz.deb('newstats', $scope.categoryStats);
			} else {
				// _oz.deb('currentstats', $scope.categoryStats);
			}

			_.each($scope.lookups.category.simpleTree, function(cat) {
				numCat = numCat + cat.subCategoryList.length;
			});

			$scope.allNote = numCat + ' categories';

			_.each($scope.categoryStats.topCategoryList, function(catId) {
				var note, catImage = _oz.setting.host();
				var category = $scope.lookups.category.idToCategory[catId];

				if (category.Image) {
					catImage = catImage + category.Image;
				} else {
					catImage = catImage + $scope.lookups.category.idToCategory[category.parentCatId].Image;
				}

				if (category.parentCatId) {
					note = 'in ' + $scope.lookups.category.idToCategory[category.parentCatId].Description;
				} else {
					note = _.find($scope.lookups.category.simpleTree, {id: catId}).subCategoryList.length + ' categories';
				}

				$scope.quickCat.push({
					description: category.Description,
					id: category.ID,
					note: note,
					image: catImage
				});
			});

			_oz.deb('quickcat', $scope.quickCat);

			//
			// Get all spotlight eligible ads
			//
			$scope.allSpotlightAds = _.filter($scope.lookups.minimalIndex, {S: 'Y'});

			//
			// Sort by Date Active and get the last 24 - effectively the newest 24 in the list, but in orde of oldest -> newest
			//
			initialSpotlightMatch = _.sortBy($scope.allSpotlightAds, 'DA');
			initialSpotlightMatch = _.last(initialSpotlightMatch, $scope.maxSpotlight);
			initialSpotlightMatch.reverse();

			DataSvc.PopulateMediumDetail(initialSpotlightMatch).then(function(result) {
				$scope.spotlightMatch = initialSpotlightMatch;
			});

		}, function(reason) {
	    	_oz.deb('$q TopCtrl error', reason);

			$ionicLoading.hide();

	    	$scope.fatalError = true;
	    });

    };

    now = new Date;

    lastRefresh = new Date(_oz.setting.lastRefresh());

    if (((now.getTime() - lastRefresh.getTime()) / 60000) > 15) {
    	$scope.forceReload();
    } else {
	    $scope.retrieveData();
    }

});