//
// Cordova plugins custom url handler
//
function handleOpenURL(url) {

    var body = document.getElementsByTagName("body")[0];
    var mainController = angular.element(body).scope();
    mainController.reportAppLaunched(url);
}

//
// Add my utils, namespaced under _oz
//
var _oz = {
  setting: {
    currentSettings: {
      env: 'PROD',
      defPageLength: 12,
      defSpotlight: 12,
      defImageQuality: 65,
      defImageWidth: 800,
      defImageHeight: 800,
      username: '',
      lastRefresh: 0
    },
    defImgSrc: function() {
      return 'img/no-image.png'
    },
    restoreSettings: function() {
      var savedSettings = angular.fromJson(window.localStorage['SAVEDSETTINGS'] || '{}');

      _.assign(this.currentSettings, savedSettings);

      return this.currentSettings;
    },
    saveSettings: function() {
      window.localStorage['SAVEDSETTINGS'] = angular.toJson(this.currentSettings);

      return this.currentSettings;
    },
    resetSettings: function() {
      window.localStorage.removeItem('SAVEDSETTINGS');

      return this.currentSettings;
    },
    host: function() {
      return this.envList()[this.env()];
    },
    dataUrl: function() {
      return this.host() + '/jsonif.aspx';
    },
    appId: function() {
      return 'JTI';
    },
    version: function() {
      return '20141014-1322-0.0.10'
    },
    env: function(newEnv) {
      if (newEnv) {
        this.currentSettings.env = newEnv;
        this.saveSettings();
      }

      return this.currentSettings.env;
    },
    lastRefresh: function(newRefresh) {
      if (newRefresh) {
        this.currentSettings.lastRefresh = newRefresh;
        this.saveSettings();
      }

      return this.currentSettings.lastRefresh;
    },
    envList: function() {
      return {
        UAT: 'https://stage.jtinsight.com',
        PROD: 'https://www.jtinsight.com'
      };
    },
    gaCodeList: function() {
      return {
        UAT: 'UA-5154602-23',
        PROD: 'UA-5154602-24'
      }
    },
    gaCode: function() {
      return this.gaCodeList()[this.env()];
    },
    postHeaders: function() {
      return {'Content-Type': 'application/x-www-form-urlencoded'};
    },
    defPageLength: function(newPageLength) {
      if (newPageLength) {
        this.currentSettings.defPageLength = newPageLength;
        this.saveSettings();
      }
      return this.currentSettings.defPageLength;
    },
    defSpotlight: function(newSpotlight) {
      if (newSpotlight) {
        this.currentSettings.defSpotlight = newSpotlight;
        this.saveSettings();
      }
      return this.currentSettings.defSpotlight;
    },
    defImageQuality: function(newImageQuality) {
      if (newImageQuality) {
        this.currentSettings.defImageQuality = newImageQuality;
        this.saveSettings();
      }

      return this.currentSettings.defImageQuality;
    },
    defImageWidth: function(newImageWidth) {
      if (newImageWidth) {
        this.currentSettings.defImageWidth = newImageWidth;
        this.saveSettings();
      }
      return this.currentSettings.defImageWidth;
    },
    defImageHeight: function(newImageHeight) {
      if (newImageHeight) {
        this.currentSettings.defImageHeight = newImageHeight;
        this.saveSettings();
      }
      return this.currentSettings.defImageHeight;
    },
    showTips: function() {
      return true;
    },
    instantSearchDebounce: function() {
      return 200;
    },
    username: function(newUsername) {
      if (newUsername) {
        this.currentSettings.username = newUsername;
        this.saveSettings();
      }

      return this.currentSettings.username;
    }
  },
  tView: function(screenTitle) {
    if (typeof analytics !== 'undefined'){
      try {
        analytics.trackView(screenTitle);
      } catch(e) {
        
      }
    }
  },
  tEvent: function(evCat, evAction, evLabel, evValue) {
    if (typeof analytics !== 'undefined'){
      try {
        analytics.trackEvent(evCat, evAction, evLabel, evValue);
      } catch (e) {

      }
    }
  },
  waiting: function(text, delay) {
    delay = delay || 250;

    return {
      template: '<i class="button-icon icon ion-loading-a"></i><br> ' + text + '...',
      delay:delay
    }
  },
  absUrl: function(sourceUrl) {
    if (sourceUrl.indexOf('http') === 0) {
      return sourceUrl;
    } else {
      return this.setting.host() + sourceUrl;
    }
  },
  deb: function() {
    var i = 0, theArg, useObject = false;

    if (arguments[0] === true) {
      useObject = true;
      i = 1;
    }

    for (;i < arguments.length; i = i + 1) {
      theArg = arguments[i];
      if (typeof theArg === 'object') {
        try {
          if (useObject) {
            console.log(theArg);
          } else {
            console.log(angular.toJson(theArg));
          }
        } catch (e) {
          console.log('toJson failed ' + theArg);
        }
      } else {
        console.log(theArg);
      }
    }
  },
  toJson: function(sourceObject) {
    return encodeURIComponent(angular.toJson(sourceObject));
  },
  fromJson: function(sourceData) {
    return angular.fromJson(decodeURIComponent(sourceData));
  },
  fromWebParams: function(url) {
    var appParams = _.clone(url);

    var searchParamList = appParams.SearchParameters.split('/');

    //
    // 0 is the url for the route
    // 1 is the name of any list, i.e _CAROUSEL
    // 2 is any search term
    // 3 is the primary category - which may be a main or a sub category
    // 4 is a list of additional filters which vary according to the main category code
    //   these are a stringified json object which is converted here
    // 5 is the sort order
    // 6 is the page length
    // 7 is the page number to start
    //
    // For the purposes of the app, we only want a subset of the above
    // paging etc. is handled differently
    //
    appParams.appSearchParameters = {
      term: searchParamList[2] === '_' ? '' : searchParamList[2],
      catPrimary: searchParamList[3],
      filter: searchParamList[4] !== '_' ? angular.fromJson(searchParamList[4]) : {},
      sort: searchParamList[5] !== '_' ? searchParamList[5] : ''
    };

    return appParams;
  },
  adNote: function(ad) {
    var locationText;
    if (ad.VYER) {
      return ad.VYER + ' reg ' + ((ad.VMIL) ? ', ' + ad.VMIL + ' mls' : '');
    }

    if (ad.catMain.Code === 'JOBS') {
      if (ad.JCON) {
        return ad.JCON + ', ' + (ad.JNAT ? 'Full-time' : 'Part-time');
      } else {
        return (ad.JNAT ? 'Full-time' : 'Part-time');
      }
    }

    if (ad.PBED) {
      locationText = '';
      if (ad.PLOC) {
        locationText = ad.PLOC;
        locationText = locationText.replace('Jersey: ', '');
        locationText = locationText.replace('Guernsey: ', '');
        locationText = locationText + ': ';
      }

      return locationText + ad.PBED + ' bed' + ((ad.PBAT) ? ', ' + ad.PBAT + ' bath' : '');
    }

    return 'created ' + ad.DA.substring(8, 10) + '/' + ad.DA.substring(5, 7);
  },
  shuffle: function(a,b){for(b=a.length;b;)a.push(a.splice(Math.random()*b--,1)[0])},
  postData: function(data) {
    var param = function(obj) {
        var query = '';
        var name, value, fullSubName, subName, subValue, innerObj, i;

        for (name in obj) {
            if (obj.hasOwnProperty(name)) {
                value = obj[name];

                if (value instanceof Array) {
                    for(i=0; i<value.length; i = i + 1) {
                        subValue = value[i];
                        fullSubName = name + '[' + i + ']';
                        innerObj = {};
                        innerObj[fullSubName] = subValue;
                        query += param(innerObj) + '&';
                    }
                } else if(value instanceof Object) {
                    for(subName in value) {
                        if (value.hasOwnProperty(subName)) {
                            subValue = value[subName];
                            fullSubName = name + '[' + subName + ']';
                            innerObj = {};
                            innerObj[fullSubName] = subValue;
                            query += param(innerObj) + '&';
                        }
                    }
                } else if(value !== undefined && value !== null) {
                    query += encodeURIComponent(name) + '=' + encodeURIComponent(value) + '&';
                }
            }
        }

        return query.length ? query.substr(0, query.length - 1) : query;
    };

    return angular.isObject(data) && String(data) !== '[object File]' ? param(data) : data;
  },
  dateTime: function(source, property) {
    var DateTimeParts, DateParts, TimeParts, NewDate;
    var original = (property) ? source[property] : source;

    if (original) {
        DateTimeParts = original.split(' ');

        DateParts = DateTimeParts[0].split('-');

        if (DateTimeParts.length > 1) {
            TimeParts = DateTimeParts[1].split(':');

            NewDate = new Date(DateParts[0], DateParts[1] - 1, DateParts[2], TimeParts[0], TimeParts[1], 0, 0);
        } else {
            NewDate = new Date(DateParts[0], DateParts[1] - 1, DateParts[2], 0, 0, 0, 0);
        }

        return NewDate;
    } else {
        return new Date(2010, 0, 1);
    }
  },
  join: function(sourceArray, joinString) {
    var filteredArray = _.filter(sourceArray, function(entry) {
      if (entry && entry.trim()) {
        return true;
      } else {
        return false;
      }
    });

    joinString = joinString || ', ';

    return filteredArray.join(joinString);
  },
  elem: function(elementName, source) {
    return '<' + elementName + '>' + source + '</' + elementName + '>'
  },
  def: function(source, defValue) {
    if (source && source !== 0 && (source + '').trim() !== '') {
      return source;
    } else {
      return defValue;
    }
  },
  assembleAd: function(editableData, categoryCode, pendingRate) {
    //
    // Transform from editable structure back into flat saveable structure and save
    //
    // Common details
    //
    var saveAd = {
      ID: editableData.Common.ID,
      Title: editableData.Common.Title,
      Description: editableData.Common.Description,
      Email: editableData.Common.Email,
      Mobile: editableData.Common.Mobile,
      Phone: editableData.Common.Phone,
      Price: editableData.Common.Price,
      JTIRAPricing: {
        pendingRate: pendingRate
      }
    } ;

    //
    // Ad type specific
    //
    if (categoryCode === 'PROPERTY') {
      _.merge(saveAd, {
        Bedrooms: editableData.Property.Bedrooms,
        Bathrooms: editableData.Property.Bathrooms,
        Livingrooms: editableData.Property.Livingrooms,
        ParkingSpaces: editableData.Property.ParkingSpaces,
        Location: editableData.Property.Parish,
        Qualification: editableData.Property.Qualification,
        Tenure: editableData.Property.Tenure
      });
    }
    
    if (categoryCode === 'JOBS') {
      _.merge(saveAd, {
        Salary: editableData.Jobs.Salary,
        JobLocation: editableData.Jobs.Location,
        Agency: editableData.Jobs.AgencyDetails,
        JobRef: editableData.Jobs.ReferenceNumber,
        FullTime: (editableData.Jobs.Nature === 'FullTime') ? true : false
      });
    } 

    if (categoryCode === 'VEHICLES') {
      _.merge(saveAd, {
        Make: editableData.Vehicles.Make,
        Model: editableData.Vehicles.Model,
        Year: editableData.Vehicles.Year,
        Dealer: (editableData.Vehicles.DealerOrPrivate === 'Dealer') ? true : false,
        Mileage: editableData.Vehicles.Mileage
      });
    }

    return saveAd;
  },
  loginPopup: function(theScope, params) {

    var defaultPopup = {
      templateUrl: 'templates/common/login-popup.html',
      title: 'Please login to continue',
      subTitle: 'Visit jtinsight.com for help and registration',
      scope: theScope,
      buttons: [
        { text: 'Cancel' },
        {
          text: '<b>Login</b>',
          type: 'button-positive',
          onTap: function(e) {
            if (!theScope.loginData.username && theScope.loginData.password) {
            //don't allow the user to close unless he enters wifi password
              e.preventDefault();
            } else {
              return theScope.loginData;
            }
          }
        }
      ]
    };

    params = params || {};

    _.assign(defaultPopup, params);

    return defaultPopup;
  },
  loginFailPopup: function(theScope, params) {

    var defaultPopup = {
      templateUrl: 'templates/common/login-fail-popup.html',
      title: 'Unable to login',
      subTitle: 'Incorrect email address or password',
      scope: theScope,
      buttons: [
        { text: 'Cancel' },
        {
          text: '<b>JT Insight</b>',
          type: 'button-positive',
          onTap: function(e) {
            return 'VISIT';
          }
        }
      ]
    };

    params = params || {};

    _.assign(defaultPopup, params);

    return defaultPopup;
  }
};



// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.services' is found in services.js
// 'starter.controllers' is found in controllers.js
angular.module('starter', [
  'ionic',
  'starter.controllers',
  'starter.services',
  'ngSanitize',
  'ngCordova'
])

.run(function($ionicPlatform) {

  $ionicPlatform.ready(function() {
    // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
    // for form inputs)
    if(window.cordova && window.cordova.plugins.Keyboard) {
      cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
    }
    if(window.StatusBar) {
      // org.apache.cordova.statusbar required
      StatusBar.styleDefault();
    }

    if (typeof analytics !== 'undefined'){
      analytics.startTrackerWithId(_oz.setting.gaCode());
    }
  });
})

.config(function($provide) {
  // from http://stackoverflow.com/questions/21714655/angular-js-angular-ui-router-reloading-current-state-refresh-data
    $provide.decorator('$state', function($delegate, $stateParams) {
        $delegate.forceReload = function() {
            return $delegate.go($delegate.current, $stateParams, {
                reload: true,
                inherit: false,
                notify: true
            });
        };
        return $delegate;
    });
})

.config(['$httpProvider', function ($httpProvider) {
    // Sean - these changes required to allow the hosted spa to receive
    // data from the WS at Insight

    // delete header from client:
    // http://stackoverflow.com/questions/17289195/angularjs-post-data-to-external-rest-api
    $httpProvider.defaults.useXDomain = true;
    $httpProvider.defaults.withCredentials = true;
    delete $httpProvider.defaults.headers.common['X-Requested-With'];
    delete $httpProvider.defaults.headers.post['Content-Type'];
}])

.config(['$stateProvider', '$urlRouterProvider', function($stateProvider, $urlRouterProvider) {

  // if none of the above states are matched, use this as the fallback
  $urlRouterProvider.otherwise('/top');

  // Ionic uses AngularUI Router which uses the concept of states
  // Learn more here: https://github.com/angular-ui/ui-router
  // Set up the various states which the app can be in.
  // Each state's controller can be found in controllers.js
  $stateProvider



    .state('top', {
      url: '/top',
      templateUrl: 'templates/top.html',
      controller: 'TopCtrl'
    })


    .state('login', {
      url: '/top:destination',
      templateUrl: 'templates/myaccount/login.html',
      controller: 'LoginCtrl'
    })


    .state('choosemaincat', {
      url: '/choosemaincat',
      templateUrl: 'templates/search/browse-choosemaincat.html',
      controller: 'BrowseMainCtrl'
    })

    .state('choosesubcat', {
      url: '/choosesubcat/:mainCatId',
      templateUrl: 'templates/search/browse-choosesubcat.html',
      controller: 'BrowseSubCtrl'
    })



    .state('myaccount', {
      url: '/myaccount',
      controller: 'MyAccountCtrl',
      templateUrl: 'templates/myaccount/myaccount.html'
    })


    .state('basket', {
      url: '/basket/:newPaymentFlowReference',
      controller: 'BasketCtrl',
      templateUrl: 'templates/post/basket.html'
    })


    .state('edit', {
      url: '/edit/:epid',
      controller: 'EditCtrl',
      templateUrl: 'templates/post/edit.html'
    })



    .state('searchresults', {
      url: '/searchresults/:contextId',
      templateUrl: 'templates/search/searchresults.html',
      controller: 'SearchResultsCtrl'
    })

    .state('detailedad', {
      url: '/detailedad/:contextId',
      templateUrl: 'templates/search/detailedad.html',
      controller: 'DetailedAdCtrl'
    })

    .state('pictureviewer', {
      url: '/pictureviewer/:epid',
      templateUrl: 'templates/search/pictureviewer.html',
      controller: 'PictureViewerCtrl'
    })



    .state('post-quickcatfilter', {
      url: '/post-quickcatfilter',
      templateUrl: 'templates/post/post-quickcatfilter.html',
      controller: 'PostCtrl'
    })

    .state('post-choosemaincat', {
      url: '/post-choosemaincat',
      templateUrl: 'templates/post/post-choosemaincat.html',
      controller: 'PostCtrl'
    })

    .state('post-choosesubcat', {
      url: '/post-choosesubcat/:mainCatId',
      templateUrl: 'templates/post/post-choosesubcat.html',
      controller: 'PostCtrl'
    })

    .state('post-quickdetails', {
      url: '/post-quickdetails/:mainCatId/:subCatId',
      templateUrl: 'templates/post/post-quickdetails.html',
      controller: 'PostQuickCtrl'
    })



    .state('publish-confirm', {
      url: '/confirm/:epid',
      templateUrl: 'templates/post/publish-confirm.html',
      controller: 'PublishCtrl'
    })



    .state('settings', {
      url: '/main',
      controller: 'SettingsCtrl',
      templateUrl: 'templates/settings/settings-main.html'
    })

}])
.filter('drama', [function() {
    var dramaRegex = new RegExp('([^0-9])(?=\\1{2,})', "g");

    return function(original) {
        var newString;

        if (!original) {
            return original;
        }

        //
        // Filter more than two duplicates of any character
        //
        newString = original.replace(dramaRegex, '');

        return newString;
    };

}])
.filter('safeDescription', ['$sanitize', function($sanitize) {

  return function(original) {
        //
        // Retain HTML formatting, but attempt to clean rogue unicode (as filter?)
        //
        var cleanDescription;
        var oldArray, newArray;
        var safeDescription;

        if (!original) {
          return original;
        }

        cleanDescription = original.replace(String.fromCharCode(65533), " ");

        //
        // Attempt to crush unicode replacements
        //
        oldArray = cleanDescription.split('');
        newArray = [];

        _.each(oldArray, function(thechar){
            if (thechar.charCodeAt(0) !== 65533) {
                newArray.push(thechar);
            } else {
                newArray.push(' ');
            }
        });

        cleanDescription = newArray.join('');

        //
        // Attempt to preserve line-breaks
        //
        cleanDescription = cleanDescription.replace(/\r\n/g, '<br/>');
        cleanDescription = cleanDescription.replace(/\n/g, '<br/>');

        //
        // Bulk properties (mostly) may have multiple <br/> tags (presumably for spacing elsewhere)
        // Limit to two at a time.
        //
        cleanDescription = cleanDescription.replace(/(<br\/>){2,}/gi, '<br/><br/>');
        //cleanDescription = $filter('linky')(cleanDescription, '_blank');

        safeDescription = $sanitize(cleanDescription);// $sce.trustAsHtml(cleanDescription);

        return safeDescription;

    };
}])
.filter('currencyInt', [ '$filter', '$locale', function ($filter, $locale) {
    var currency = $filter('currency'), formats = $locale.NUMBER_FORMATS, value;
    return function (amount, symbol) {
        value = currency(amount, symbol);
        return value.replace(new RegExp('\\' + formats.DECIMAL_SEP + '\\d{2}'), '');
    };
}])
.filter('adRate', [ '$filter', '$locale', function ($filter, $locale) {
    var currency = $filter('currency'), value;
    return function (amount, symbol) {
        if (amount === 0) {
            return 'FREE';
        }

        value = currency(amount, symbol);
        return value;
    };
}])
.directive('ngEnter', function () {
    return function (scope, element, attrs) {
        element.bind("keydown keypress", function (event) {
            if(event.which === 13) {
                scope.$apply(function (){
                    scope.$eval(attrs.ngEnter);
                });
 
                event.preventDefault();
            }
        });
    };
})
.directive('errSrc', function() {
  return {
    link: function(scope, element, attrs) {

      scope.$watch(function() {
          return attrs['ngSrc'];
        }, function (value) {
          if (!value) {
            if (attrs.errSrc) {
              element.attr('src', attrs.errSrc);  
            } else {
              element.attr('src', _oz.setting.defImgSrc());
            }
          }
      });

      element.bind('error', function() {
        if (attrs.errSrc) {
        element.attr('src', attrs.errSrc);
      } else {
              element.attr('src', _oz.setting.defImgSrc());
      }
      });
    }
  }
})
;

